@@ -20,8 +20,8 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <kdrive-config.h>
+#ifdef HAVE_DIX_CONFIG_H
+#include <dix-config.h>
 #endif
 #include "kdrive.h"
 #include <mivalidate.h>
@@ -30,6 +30,7 @@
 #ifdef RANDR
 #include <randrstr.h>
 #endif
+#include "glx_extinit.h"
 
 #ifdef XV
 #include "kxv.h"
@@ -43,7 +44,13 @@
 #include <execinfo.h>
 #endif
 
-#include <signal.h>
+#if defined(CONFIG_UDEV) || defined(CONFIG_HAL)
+#include <hotplug.h>
+#endif
+
+/* This stub can be safely removed once we can
+ * split input and GPU parts in hotplug.h et al. */
+#include <systemd-logind.h>
 
 typedef struct _kdDepths {
     CARD8 depth;
@@ -60,36 +67,25 @@ KdDepths kdDepths[] = {
     {32, 32}
 };
 
-#define NUM_KD_DEPTHS (sizeof (kdDepths) / sizeof (kdDepths[0]))
-
 #define KD_DEFAULT_BUTTONS 5
 
 DevPrivateKeyRec kdScreenPrivateKeyRec;
-unsigned long kdGeneration;
+static unsigned long kdGeneration;
 
-Bool kdVideoTest;
-unsigned long kdVideoTestTime;
 Bool kdEmulateMiddleButton;
 Bool kdRawPointerCoordinates;
 Bool kdDisableZaphod;
-Bool kdAllowZap;
-Bool kdEnabled;
-int kdSubpixelOrder;
-int kdVirtualTerminal = -1;
-Bool kdSwitchPending;
-char *kdSwitchCmd;
-DDXPointRec kdOrigin;
+static Bool kdEnabled;
+static int kdSubpixelOrder;
+static char *kdSwitchCmd;
+static DDXPointRec kdOrigin;
 Bool kdHasPointer = FALSE;
 Bool kdHasKbd = FALSE;
-
-static Bool kdCaughtSignal = FALSE;
-
-/*
- * Carry arguments from InitOutput through driver initialization
- * to KdScreenInit
- */
-
-KdOsFuncs *kdOsFuncs;
+const char *kdGlobalXkbRules = NULL;
+const char *kdGlobalXkbModel = NULL;
+const char *kdGlobalXkbLayout = NULL;
+const char *kdGlobalXkbVariant = NULL;
+const char *kdGlobalXkbOptions = NULL;
 
 void
 KdDisableScreen(ScreenPtr pScreen)
@@ -103,14 +99,7 @@ KdDisableScreen(ScreenPtr pScreen)
     KdDisableColormap(pScreen);
     if (!pScreenPriv->screen->dumb && pScreenPriv->card->cfuncs->disableAccel)
         (*pScreenPriv->card->cfuncs->disableAccel) (pScreen);
-    if (!pScreenPriv->screen->softCursor &&
-        pScreenPriv->card->cfuncs->disableCursor)
-        (*pScreenPriv->card->cfuncs->disableCursor) (pScreen);
-    if (pScreenPriv->card->cfuncs->dpms)
-        (*pScreenPriv->card->cfuncs->dpms) (pScreen, KD_DPMS_NORMAL);
     pScreenPriv->enabled = FALSE;
-    if (pScreenPriv->card->cfuncs->disable)
-        (*pScreenPriv->card->cfuncs->disable) (pScreen);
 }
 
 static void
@@ -133,7 +122,7 @@ KdDoSwitchCmd(const char *reason)
     }
 }
 
-void
+static void
 KdSuspend(void)
 {
     KdCardInfo *card;
@@ -144,23 +133,17 @@ KdSuspend(void)
             for (screen = card->screenList; screen; screen = screen->next)
                 if (screen->mynum == card->selected && screen->pScreen)
                     KdDisableScreen(screen->pScreen);
-            if (card->driver && card->cfuncs->restore)
-                (*card->cfuncs->restore) (card);
         }
         KdDisableInput();
         KdDoSwitchCmd("suspend");
     }
 }
 
-void
+static void
 KdDisableScreens(void)
 {
     KdSuspend();
-    if (kdEnabled) {
-        if (kdOsFuncs->Disable)
-            (*kdOsFuncs->Disable) ();
-        kdEnabled = FALSE;
-    }
+    kdEnabled = FALSE;
 }
 
 Bool
@@ -170,78 +153,20 @@ KdEnableScreen(ScreenPtr pScreen)
 
     if (pScreenPriv->enabled)
         return TRUE;
-    if (pScreenPriv->card->cfuncs->enable)
-        if (!(*pScreenPriv->card->cfuncs->enable) (pScreen))
-            return FALSE;
     pScreenPriv->enabled = TRUE;
     pScreenPriv->dpmsState = KD_DPMS_NORMAL;
     pScreenPriv->card->selected = pScreenPriv->screen->mynum;
-    if (!pScreenPriv->screen->softCursor &&
-        pScreenPriv->card->cfuncs->enableCursor)
-        (*pScreenPriv->card->cfuncs->enableCursor) (pScreen);
     if (!pScreenPriv->screen->dumb && pScreenPriv->card->cfuncs->enableAccel)
         (*pScreenPriv->card->cfuncs->enableAccel) (pScreen);
     KdEnableColormap(pScreen);
     SetRootClip(pScreen, ROOT_CLIP_FULL);
-    if (pScreenPriv->card->cfuncs->dpms)
-        (*pScreenPriv->card->cfuncs->dpms) (pScreen, pScreenPriv->dpmsState);
     return TRUE;
 }
 
-void
-KdResume(void)
-{
-    KdCardInfo *card;
-    KdScreenInfo *screen;
-
-    if (kdEnabled) {
-        KdDoSwitchCmd("resume");
-        for (card = kdCardInfo; card; card = card->next) {
-            if (card->cfuncs->preserve)
-                (*card->cfuncs->preserve) (card);
-            for (screen = card->screenList; screen; screen = screen->next)
-                if (screen->mynum == card->selected && screen->pScreen)
-                    KdEnableScreen(screen->pScreen);
-        }
-        KdEnableInput();
-        KdReleaseAllKeys();
-    }
-}
-
-void
-KdEnableScreens(void)
-{
-    if (!kdEnabled) {
-        kdEnabled = TRUE;
-        if (kdOsFuncs->Enable)
-            (*kdOsFuncs->Enable) ();
-    }
-    KdResume();
-}
-
-void
-KdProcessSwitch(void)
-{
-    if (kdEnabled)
-        KdDisableScreens();
-    else
-        KdEnableScreens();
-}
-
 void
 AbortDDX(enum ExitCode error)
 {
     KdDisableScreens();
-    if (kdOsFuncs) {
-        if (kdEnabled && kdOsFuncs->Disable)
-            (*kdOsFuncs->Disable) ();
-        if (kdOsFuncs->Fini)
-            (*kdOsFuncs->Fini) ();
-        KdDoSwitchCmd("stop");
-    }
-
-    if (kdCaughtSignal)
-        OsAbort();
 }
 
 void
@@ -250,8 +175,8 @@ ddxGiveUp(enum ExitCode error)
     AbortDDX(error);
 }
 
-Bool kdDumbDriver;
-Bool kdSoftCursor;
+static Bool kdDumbDriver;
+static Bool kdSoftCursor;
 
 const char *
 KdParseFindNext(const char *cur, const char *delim, char *save, char *last)
@@ -407,18 +332,7 @@ KdParseScreen(KdScreenInfo * screen, const char *arg)
     }
 }
 
-/*
- * Mouse argument syntax:
- *
- *  device,protocol,options...
- *
- *  Options are any of:
- *	1-5	    n button mouse
- *	2button	    emulate middle button
- *	{NMO}	    Reorder buttons
- */
-
-void
+static void
 KdParseRgba(char *rgba)
 {
     if (!strcmp(rgba, "rgb"))
@@ -447,6 +361,11 @@ KdUseMsg(void)
         ("-mouse driver [,n,,options]    Specify the pointer driver and its options (n is the number of buttons)\n");
     ErrorF
         ("-keybd driver [,,options]      Specify the keyboard driver and its options\n");
+    ErrorF("-xkb-rules       Set default XkbRules value (can be overriden by -keybd options)\n");
+    ErrorF("-xkb-model       Set default XkbModel value (can be overriden by -keybd options)\n");
+    ErrorF("-xkb-layout      Set default XkbLayout value (can be overriden by -keybd options)\n");
+    ErrorF("-xkb-variant     Set default XkbVariant value (can be overriden by -keybd options)\n");
+    ErrorF("-xkb-options     Set default XkbOptions value (can be overriden by -keybd options)\n");
     ErrorF("-zaphod          Disable cursor screen switching\n");
     ErrorF("-2button         Emulate 3 button mouse\n");
     ErrorF("-3button         Disable 3 button mouse emulation\n");
@@ -458,7 +377,6 @@ KdUseMsg(void)
     ErrorF
         ("-origin X,Y      Locates the next screen in the the virtual screen (Xinerama)\n");
     ErrorF("-switchCmd       Command to execute on vt switch\n");
-    ErrorF("-zap             Terminate server on Ctrl+Alt+Backspace\n");
     ErrorF
         ("vtxx             Use virtual terminal xx instead of the next available\n");
 }
@@ -491,10 +409,6 @@ KdProcessArgument(int argc, char **argv, int i)
         kdDisableZaphod = TRUE;
         return 1;
     }
-    if (!strcmp(argv[i], "-zap")) {
-        kdAllowZap = TRUE;
-        return 1;
-    }
     if (!strcmp(argv[i], "-3button")) {
         kdEmulateMiddleButton = FALSE;
         return 1;
@@ -515,10 +429,6 @@ KdProcessArgument(int argc, char **argv, int i)
         kdSoftCursor = TRUE;
         return 1;
     }
-    if (!strcmp(argv[i], "-videoTest")) {
-        kdVideoTest = TRUE;
-        return 1;
-    }
     if (!strcmp(argv[i], "-origin")) {
         if ((i + 1) < argc) {
             char *x = argv[i + 1];
@@ -551,9 +461,45 @@ KdProcessArgument(int argc, char **argv, int i)
             UseMsg();
         return 2;
     }
-    if (!strncmp(argv[i], "vt", 2) &&
-        sscanf(argv[i], "vt%2d", &kdVirtualTerminal) == 1) {
-        return 1;
+    if (!strcmp(argv[i], "-xkb-rules")) {
+        if (i + 1 >= argc) {
+            UseMsg();
+            FatalError("Missing argument for option -xkb-rules.\n");
+        }
+        kdGlobalXkbRules = argv[i + 1];
+        return 2;
+    }
+    if (!strcmp(argv[i], "-xkb-model")) {
+        if (i + 1 >= argc) {
+            UseMsg();
+            FatalError("Missing argument for option -xkb-model.\n");
+        }
+        kdGlobalXkbModel = argv[i + 1];
+        return 2;
+    }
+    if (!strcmp(argv[i], "-xkb-layout")) {
+        if (i + 1 >= argc) {
+            UseMsg();
+            FatalError("Missing argument for option -xkb-layout.\n");
+        }
+        kdGlobalXkbLayout = argv[i + 1];
+        return 2;
+    }
+    if (!strcmp(argv[i], "-xkb-variant")) {
+        if (i + 1 >= argc) {
+            UseMsg();
+            FatalError("Missing argument for option -xkb-variant.\n");
+        }
+        kdGlobalXkbVariant = argv[i + 1];
+        return 2;
+    }
+    if (!strcmp(argv[i], "-xkb-options")) {
+        if (i + 1 >= argc) {
+            UseMsg();
+            FatalError("Missing argument for option -xkb-options.\n");
+        }
+        kdGlobalXkbOptions = argv[i + 1];
+        return 2;
     }
     if (!strcmp(argv[i], "-mouse") || !strcmp(argv[i], "-pointer")) {
         if (i + 1 >= argc)
@@ -573,25 +519,7 @@ KdProcessArgument(int argc, char **argv, int i)
     return 0;
 }
 
-/*
- * These are getting tossed in here until I can think of where
- * they really belong
- */
-
-void
-KdOsInit(KdOsFuncs * pOsFuncs)
-{
-    kdOsFuncs = pOsFuncs;
-    if (pOsFuncs) {
-        if (serverGeneration == 1) {
-            KdDoSwitchCmd("start");
-            if (pOsFuncs->Init)
-                (*pOsFuncs->Init) ();
-        }
-    }
-}
-
-Bool
+static Bool
 KdAllocatePrivates(ScreenPtr pScreen)
 {
     KdPrivScreenPtr pScreenPriv;
@@ -609,7 +537,7 @@ KdAllocatePrivates(ScreenPtr pScreen)
     return TRUE;
 }
 
-Bool
+static Bool
 KdCreateScreenResources(ScreenPtr pScreen)
 {
     KdScreenPriv(pScreen);
@@ -628,7 +556,7 @@ KdCreateScreenResources(ScreenPtr pScreen)
     return ret;
 }
 
-Bool
+static Bool
 KdCloseScreen(ScreenPtr pScreen)
 {
     KdScreenPriv(pScreen);
@@ -647,26 +575,12 @@ KdCloseScreen(ScreenPtr pScreen)
     else
         ret = TRUE;
 
-    if (pScreenPriv->dpmsState != KD_DPMS_NORMAL)
-        (*card->cfuncs->dpms) (pScreen, KD_DPMS_NORMAL);
-
     if (screen->mynum == card->selected)
         KdDisableScreen(pScreen);
 
-    /*
-     * Restore video hardware when last screen is closed
-     */
-    if (screen == card->screenList) {
-        if (kdEnabled && card->cfuncs->restore)
-            (*card->cfuncs->restore) (card);
-    }
-
     if (!pScreenPriv->screen->dumb && card->cfuncs->finiAccel)
         (*card->cfuncs->finiAccel) (pScreen);
 
-    if (!pScreenPriv->screen->softCursor && card->cfuncs->finiCursor)
-        (*card->cfuncs->finiCursor) (pScreen);
-
     if (card->cfuncs->scrfini)
         (*card->cfuncs->scrfini) (screen);
 
@@ -681,11 +595,7 @@ KdCloseScreen(ScreenPtr pScreen)
          * Clean up OS when last card is closed
          */
         if (card == kdCardInfo) {
-            if (kdEnabled) {
-                kdEnabled = FALSE;
-                if (kdOsFuncs->Disable)
-                    (*kdOsFuncs->Disable) ();
-            }
+            kdEnabled = FALSE;
         }
     }
 
@@ -695,37 +605,10 @@ KdCloseScreen(ScreenPtr pScreen)
     return ret;
 }
 
-Bool
+static Bool
 KdSaveScreen(ScreenPtr pScreen, int on)
 {
-    KdScreenPriv(pScreen);
-    int dpmsState;
-
-    if (!pScreenPriv->card->cfuncs->dpms)
-        return FALSE;
-
-    dpmsState = pScreenPriv->dpmsState;
-    switch (on) {
-    case SCREEN_SAVER_OFF:
-        dpmsState = KD_DPMS_NORMAL;
-        break;
-    case SCREEN_SAVER_ON:
-        if (dpmsState == KD_DPMS_NORMAL)
-            dpmsState = KD_DPMS_NORMAL + 1;
-        break;
-    case SCREEN_SAVER_CYCLE:
-        if (dpmsState < KD_DPMS_MAX)
-            dpmsState++;
-        break;
-    case SCREEN_SAVER_FORCER:
-        break;
-    }
-    if (dpmsState != pScreenPriv->dpmsState) {
-        if (pScreenPriv->enabled)
-            (*pScreenPriv->card->cfuncs->dpms) (pScreen, dpmsState);
-        pScreenPriv->dpmsState = dpmsState;
-    }
-    return TRUE;
+    return FALSE;
 }
 
 static Bool
@@ -802,7 +685,7 @@ KdSetSubpixelOrder(ScreenPtr pScreen, Rotation randr)
 /* Pass through AddScreen, which doesn't take any closure */
 static KdScreenInfo *kdCurrentScreen;
 
-Bool
+static Bool
 KdScreenInit(ScreenPtr pScreen, int argc, char **argv)
 {
     KdScreenInfo *screen = kdCurrentScreen;
@@ -912,7 +795,6 @@ KdScreenInit(ScreenPtr pScreen, int argc, char **argv)
     /*
      * Wrap CloseScreen, the order now is:
      *  KdCloseScreen
-     *  miBSCloseScreen
      *  fbCloseScreen
      */
     pScreenPriv->CloseScreen = pScreen->CloseScreen;
@@ -937,21 +819,10 @@ KdScreenInit(ScreenPtr pScreen, int argc, char **argv)
     /*
      * Enable the hardware
      */
-    if (!kdEnabled) {
-        kdEnabled = TRUE;
-        if (kdOsFuncs->Enable)
-            (*kdOsFuncs->Enable) ();
-    }
+    kdEnabled = TRUE;
 
     if (screen->mynum == card->selected) {
-        if (card->cfuncs->preserve)
-            (*card->cfuncs->preserve) (card);
-        if (card->cfuncs->enable)
-            if (!(*card->cfuncs->enable) (pScreen))
-                return FALSE;
         pScreenPriv->enabled = TRUE;
-        if (!screen->softCursor && card->cfuncs->enableCursor)
-            (*card->cfuncs->enableCursor) (pScreen);
         KdEnableColormap(pScreen);
         if (!screen->dumb && card->cfuncs->enableAccel)
             (*card->cfuncs->enableAccel) (pScreen);
@@ -960,7 +831,7 @@ KdScreenInit(ScreenPtr pScreen, int argc, char **argv)
     return TRUE;
 }
 
-void
+static void
 KdInitScreen(ScreenInfo * pScreenInfo,
              KdScreenInfo * screen, int argc, char **argv)
 {
@@ -1009,7 +880,7 @@ KdSetPixmapFormats(ScreenInfo * pScreenInfo)
     /*
      * Fill in additional formats
      */
-    for (i = 0; i < NUM_KD_DEPTHS; i++)
+    for (i = 0; i < ARRAY_SIZE(kdDepths); i++)
         if (!depthToBpp[kdDepths[i].depth])
             depthToBpp[kdDepths[i].depth] = kdDepths[i].bpp;
 
@@ -1062,27 +933,6 @@ KdAddScreen(ScreenInfo * pScreenInfo,
     AddScreen(KdScreenInit, argc, argv);
 }
 
-#if 0                           /* This function is not used currently */
-
-int
-KdDepthToFb(ScreenPtr pScreen, int depth)
-{
-    KdScreenPriv(pScreen);
-
-    for (fb = 0; fb <= KD_MAX_FB && pScreenPriv->screen->fb.frameBuffer; fb++)
-        if (pScreenPriv->screen->fb.depth == depth)
-            return fb;
-}
-
-#endif
-
-static int
-KdSignalWrapper(int signum)
-{
-    kdCaughtSignal = TRUE;
-    return 1;                   /* use generic OS layer cleanup & abort */
-}
-
 void
 KdInitOutput(ScreenInfo * pScreenInfo, int argc, char **argv)
 {
@@ -1124,7 +974,12 @@ KdInitOutput(ScreenInfo * pScreenInfo, int argc, char **argv)
         for (screen = card->screenList; screen; screen = screen->next)
             KdAddScreen(pScreenInfo, screen, argc, argv);
 
-    OsRegisterSigWrapper(KdSignalWrapper);
+    xorgGlxCreateVendor();
+
+#if defined(CONFIG_UDEV) || defined(CONFIG_HAL)
+    if (SeatId) /* Enable input hot-plugging */
+        config_pre_init();
+#endif
 }
 
 void
@@ -1132,14 +987,35 @@ OsVendorFatalError(const char *f, va_list args)
 {
 }
 
-int
-DPMSSet(ClientPtr client, int level)
+/* These stubs can be safely removed once we can
+ * split input and GPU parts in hotplug.h et al. */
+#ifdef CONFIG_UDEV_KMS
+void
+NewGPUDeviceRequest(struct OdevAttributes *attribs)
 {
-    return Success;
 }
 
-Bool
-DPMSSupported(void)
+void
+DeleteGPUDeviceRequest(struct OdevAttributes *attribs)
 {
-    return FALSE;
 }
+#endif
+
+struct xf86_platform_device *
+xf86_find_platform_device_by_devnum(int major, int minor)
+{
+    return NULL;
+}
+
+#ifdef SYSTEMD_LOGIND
+void
+systemd_logind_vtenter(void)
+{
+}
+
+void
+systemd_logind_release_fd(int major, int minor, int fd)
+{
+    close(fd);
+}
+#endif
