@@ -50,76 +50,50 @@
 #include <xorg-config.h>
 #endif
 
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <sys/types.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <string.h>
-#include <stdarg.h>
-
 #include "os.h"
 #include "loader.h"
 #include "loaderProcs.h"
-#include "xf86.h"
-#include "xf86Priv.h"
-#include "compiler.h"
 
 #ifdef HAVE_DLFCN_H
 
 #include <dlfcn.h>
 #include <X11/Xos.h>
 
-#if defined(DL_LAZY)
-#define DLOPEN_LAZY DL_LAZY
-#elif defined(RTLD_LAZY)
-#define DLOPEN_LAZY RTLD_LAZY
-#elif defined(__FreeBSD__)
-#define DLOPEN_LAZY 1
-#else
-#define DLOPEN_LAZY 0
-#endif
-
-#if defined(LD_GLOBAL)
-#define DLOPEN_GLOBAL LD_GLOBAL
-#elif defined(RTLD_GLOBAL)
-#define DLOPEN_GLOBAL RTLD_GLOBAL
-#else
-#define DLOPEN_GLOBAL 0
-#endif
-
 #else
 #error i have no dynamic linker and i must scream
 #endif
 
+#ifndef XORG_NO_SDKSYMS
 extern void *xorg_symbols[];
+#endif
 
 void
 LoaderInit(void)
 {
-    xf86MsgVerb(X_INFO, 2, "Loader magic: %p\n", (void *) xorg_symbols);
-    xf86MsgVerb(X_INFO, 2, "Module ABI versions:\n");
-    xf86ErrorFVerb(2, "\t%s: %d.%d\n", ABI_CLASS_ANSIC,
-                   GET_ABI_MAJOR(LoaderVersionInfo.ansicVersion),
-                   GET_ABI_MINOR(LoaderVersionInfo.ansicVersion));
-    xf86ErrorFVerb(2, "\t%s: %d.%d\n", ABI_CLASS_VIDEODRV,
-                   GET_ABI_MAJOR(LoaderVersionInfo.videodrvVersion),
-                   GET_ABI_MINOR(LoaderVersionInfo.videodrvVersion));
-    xf86ErrorFVerb(2, "\t%s : %d.%d\n", ABI_CLASS_XINPUT,
-                   GET_ABI_MAJOR(LoaderVersionInfo.xinputVersion),
-                   GET_ABI_MINOR(LoaderVersionInfo.xinputVersion));
-    xf86ErrorFVerb(2, "\t%s : %d.%d\n", ABI_CLASS_EXTENSION,
-                   GET_ABI_MAJOR(LoaderVersionInfo.extensionVersion),
-                   GET_ABI_MINOR(LoaderVersionInfo.extensionVersion));
+#ifndef XORG_NO_SDKSYMS
+    LogMessageVerb(X_INFO, 2, "Loader magic: %p\n", (void *) xorg_symbols);
+#endif
+    LogMessageVerb(X_INFO, 2, "Module ABI versions:\n");
+    LogWrite(2, "\t%s: %d.%d\n", ABI_CLASS_ANSIC,
+             GET_ABI_MAJOR(LoaderVersionInfo.ansicVersion),
+             GET_ABI_MINOR(LoaderVersionInfo.ansicVersion));
+    LogWrite(2, "\t%s: %d.%d\n", ABI_CLASS_VIDEODRV,
+             GET_ABI_MAJOR(LoaderVersionInfo.videodrvVersion),
+             GET_ABI_MINOR(LoaderVersionInfo.videodrvVersion));
+    LogWrite(2, "\t%s : %d.%d\n", ABI_CLASS_XINPUT,
+             GET_ABI_MAJOR(LoaderVersionInfo.xinputVersion),
+             GET_ABI_MINOR(LoaderVersionInfo.xinputVersion));
+    LogWrite(2, "\t%s : %d.%d\n", ABI_CLASS_EXTENSION,
+             GET_ABI_MAJOR(LoaderVersionInfo.extensionVersion),
+             GET_ABI_MINOR(LoaderVersionInfo.extensionVersion));
 
 }
 
 /* Public Interface to the loader. */
 
 void *
-LoaderOpen(const char *module, int *errmaj, int *errmin)
+LoaderOpen(const char *module, int *errmaj)
 {
     void *ret;
 
@@ -127,14 +101,12 @@ LoaderOpen(const char *module, int *errmaj, int *errmin)
     ErrorF("LoaderOpen(%s)\n", module);
 #endif
 
-    xf86Msg(X_INFO, "Loading %s\n", module);
+    LogMessage(X_INFO, "Loading %s\n", module);
 
-    if (!(ret = dlopen(module, DLOPEN_LAZY | DLOPEN_GLOBAL))) {
-        xf86Msg(X_ERROR, "Failed to load %s: %s\n", module, dlerror());
+    if (!(ret = dlopen(module, RTLD_LAZY | RTLD_GLOBAL))) {
+        LogMessage(X_ERROR, "Failed to load %s: %s\n", module, dlerror());
         if (errmaj)
             *errmaj = LDR_NOLOAD;
-        if (errmin)
-            *errmin = LDR_NOLOAD;
         return NULL;
     }
 
@@ -152,7 +124,7 @@ LoaderSymbol(const char *name)
         return p;
 
     if (!global_scope)
-        global_scope = dlopen(NULL, DLOPEN_LAZY | DLOPEN_GLOBAL);
+        global_scope = dlopen(NULL, RTLD_LAZY | RTLD_GLOBAL);
 
     if (global_scope)
         return dlsym(global_scope, name);
@@ -160,10 +132,16 @@ LoaderSymbol(const char *name)
     return NULL;
 }
 
+void *
+LoaderSymbolFromModule(void *handle, const char *name)
+{
+    return dlsym(handle, name);
+}
+
 void
 LoaderUnload(const char *name, void *handle)
 {
-    xf86Msg(X_INFO, "Unloading %s\n", name);
+    LogMessageVerbSigSafe(X_INFO, 1, "Unloading %s\n", name);
     if (handle)
         dlclose(handle);
 }
@@ -189,13 +167,11 @@ LoaderGetABIVersion(const char *abiclass)
         const char *name;
         int version;
     } classes[] = {
-        {
-        ABI_CLASS_ANSIC, LoaderVersionInfo.ansicVersion}, {
-        ABI_CLASS_VIDEODRV, LoaderVersionInfo.videodrvVersion}, {
-        ABI_CLASS_XINPUT, LoaderVersionInfo.xinputVersion}, {
-        ABI_CLASS_EXTENSION, LoaderVersionInfo.extensionVersion}, {
-        ABI_CLASS_FONT, LoaderVersionInfo.fontVersion}, {
-        NULL, 0}
+        {ABI_CLASS_ANSIC, LoaderVersionInfo.ansicVersion},
+        {ABI_CLASS_VIDEODRV, LoaderVersionInfo.videodrvVersion},
+        {ABI_CLASS_XINPUT, LoaderVersionInfo.xinputVersion},
+        {ABI_CLASS_EXTENSION, LoaderVersionInfo.extensionVersion},
+        {NULL, 0}
     };
     int i;
 
