@@ -29,14 +29,11 @@
  */
 
 #include "sanitizedCarbon.h"
-#include <AvailabilityMacros.h>
 
 #ifdef HAVE_DIX_CONFIG_H
 #include <dix-config.h>
 #endif
 
-#include "quartzCommon.h"
-
 #import "X11Controller.h"
 #import "X11Application.h"
 
@@ -59,6 +56,19 @@
 extern aslclient aslc;
 extern char *bundle_id_prefix;
 
+@interface X11Controller ()
+#ifdef XQUARTZ_SPARKLE
+@property (nonatomic, readwrite, strong) NSMenuItem *check_for_updates_item; // Programatically enabled
+#endif
+
+@property (nonatomic, readwrite, strong) NSArray *apps;
+@property (nonatomic, readwrite, strong) NSMutableArray *table_apps;
+@property (nonatomic, readwrite, assign) NSInteger windows_menu_nitems;
+@property (nonatomic, readwrite, assign) int checked_window_item;
+@property (nonatomic, readwrite, assign) x_list *pending_apps;
+@property (nonatomic, readwrite, assign) OSX_BOOL finished_launching;
+@end
+
 @implementation X11Controller
 
 - (void) awakeFromNib
@@ -67,7 +77,7 @@ extern char *bundle_id_prefix;
     NSArray *array;
 
     /* Point X11Application at ourself. */
-    [xapp set_controller:self];
+    xapp.controller = self;
 
     array = [xapp prefs_get_array:@PREFS_APPSMENU];
     if (array != nil) {
@@ -103,15 +113,7 @@ extern char *bundle_id_prefix;
      addObserver: self
         selector: @selector(apps_table_done:)
             name: NSWindowWillCloseNotification
-          object: [apps_table window]];
-
-    // Setup data about our Windows menu
-    if (window_separator) {
-        [[window_separator menu] removeItem:window_separator];
-        window_separator = nil;
-    }
-
-    windows_menu_start = [[X11App windowsMenu] numberOfItems];
+          object: self.apps_table.window];
 }
 
 - (void) item_selected:sender
@@ -122,80 +124,16 @@ extern char *bundle_id_prefix;
                        AppleWMWindowMenuItem, [sender tag]);
 }
 
-- (void) remove_window_menu
-{
-    NSMenu *menu;
-    int count, i;
-
-    /* Work backwards so we don't mess up the indices */
-    menu = [X11App windowsMenu];
-    count = [menu numberOfItems];
-    for (i = count - 1; i >= windows_menu_start; i--)
-        [menu removeItemAtIndex:i];
-
-    count = [dock_menu indexOfItem:dock_window_separator];
-    for (i = 0; i < count; i++)
-        [dock_menu removeItemAtIndex:0];
-}
-
-- (void) install_window_menu:(NSArray *)list
-{
-    NSMenu *menu;
-    NSMenuItem *item;
-    int first, count, i;
-
-    menu = [X11App windowsMenu];
-    first = windows_menu_start + 1;
-    count = [list count];
-
-    // Push a Separator
-    if (count) {
-        [menu addItem:[NSMenuItem separatorItem]];
-    }
-
-    for (i = 0; i < count; i++) {
-        NSString *name, *shortcut;
-
-        name = [[list objectAtIndex:i] objectAtIndex:0];
-        shortcut = [[list objectAtIndex:i] objectAtIndex:1];
-
-        if (windowItemModMask == 0 || windowItemModMask == -1)
-            shortcut = @"";
-
-        item =
-            (NSMenuItem *)[menu addItemWithTitle:name action:
-                           @selector
-                           (item_selected:) keyEquivalent:shortcut];
-        [item setKeyEquivalentModifierMask:(NSUInteger)windowItemModMask];
-        [item setTarget:self];
-        [item setTag:i];
-        [item setEnabled:YES];
-
-        item = (NSMenuItem *)[dock_menu  insertItemWithTitle:name
-                                                      action:@selector
-                              (item_selected:) keyEquivalent:shortcut
-                                                     atIndex:i];
-        [item setKeyEquivalentModifierMask:(NSUInteger)windowItemModMask];
-        [item setTarget:self];
-        [item setTag:i];
-        [item setEnabled:YES];
-    }
-
-    if (checked_window_item >= 0 && checked_window_item < count) {
-        item = (NSMenuItem *)[menu itemAtIndex:first + checked_window_item];
-        [item setState:NSOnState];
-        item = (NSMenuItem *)[dock_menu itemAtIndex:checked_window_item];
-        [item setState:NSOnState];
-    }
-}
-
 - (void) remove_apps_menu
 {
     NSMenu *menu;
     NSMenuItem *item;
     int i;
 
-    if (apps == nil || apps_separator == nil) return;
+    NSMenuItem * const apps_separator = self.apps_separator;
+    NSMenu * const dock_apps_menu = self.dock_apps_menu;
+
+    if (self.apps == nil || apps_separator == nil) return;
 
     menu = [apps_separator menu];
 
@@ -215,8 +153,7 @@ extern char *bundle_id_prefix;
         }
     }
 
-    [apps release];
-    apps = nil;
+    self.apps = nil;
 }
 
 - (void) prepend_apps_item:(NSArray *)list index:(int)i menu:(NSMenu *)menu
@@ -253,6 +190,9 @@ extern char *bundle_id_prefix;
 
     count = [list count];
 
+    NSMenuItem * const apps_separator = self.apps_separator;
+    NSMenu * const dock_apps_menu = self.dock_apps_menu;
+
     if (count == 0 || apps_separator == nil) return;
 
     menu = [apps_separator menu];
@@ -264,28 +204,89 @@ extern char *bundle_id_prefix;
             [self prepend_apps_item:list index:i menu:dock_apps_menu];
     }
 
-    apps = [list retain];
+    self.apps = list;
 }
 
 - (void) set_window_menu:(NSArray *)list
 {
-    [self remove_window_menu];
-    [self install_window_menu:list];
+    NSMenu * const menu = X11App.windowsMenu;
+    NSMenu * const dock_menu = self.dock_menu;
 
-    DarwinSendDDXEvent(kXquartzControllerNotify, 1,
-                       AppleWMWindowMenuNotify);
+    /* First, remove the existing items from the Window Menu */
+    NSInteger itemsToRemove = self.windows_menu_nitems;
+    if (itemsToRemove > 0) {
+        NSInteger indexForRemoval = menu.numberOfItems - itemsToRemove - 1; /* we also want to remove the separator */
+
+        for (NSInteger i = 0 ; i < itemsToRemove + 1 ; i++) {
+            [menu removeItemAtIndex:indexForRemoval];
+        }
+
+        for (NSInteger i = 0 ; i < itemsToRemove; i++) {
+            [dock_menu removeItemAtIndex:0];
+        }
+    }
+
+    NSInteger const itemsToAdd = list.count;
+    self.windows_menu_nitems = itemsToAdd;
+
+    if (itemsToAdd > 0) {
+        NSMenuItem *item;
+
+        // Push a Separator
+        [menu addItem:[NSMenuItem separatorItem]];
+
+        for (NSInteger i = 0; i < itemsToAdd; i++) {
+            NSString *name, *shortcut;
+
+            name = list[i][0];
+            shortcut = list[i][1];
+
+            if (windowItemModMask == 0 || windowItemModMask == -1)
+                shortcut = @"";
+
+            item = (NSMenuItem *)[menu addItemWithTitle:name
+                                                 action:@selector(item_selected:)
+                                          keyEquivalent:shortcut];
+            [item setKeyEquivalentModifierMask:(NSUInteger)windowItemModMask];
+            [item setTarget:self];
+            [item setTag:i];
+            [item setEnabled:YES];
+
+            item = (NSMenuItem *)[dock_menu  insertItemWithTitle:name
+                                                          action:@selector(item_selected:)
+                                                   keyEquivalent:shortcut
+                                                         atIndex:i];
+            [item setKeyEquivalentModifierMask:(NSUInteger)windowItemModMask];
+            [item setTarget:self];
+            [item setTag:i];
+            [item setEnabled:YES];
+        }
+
+        int const checked_window_item = self.checked_window_item;
+        if (checked_window_item >= 0 && checked_window_item < itemsToAdd) {
+            NSInteger first = menu.numberOfItems - itemsToAdd;
+            item = (NSMenuItem *)[menu itemAtIndex:first + checked_window_item];
+            [item setState:NSOnState];
+
+            item = (NSMenuItem *)[dock_menu itemAtIndex:checked_window_item];
+            [item setState:NSOnState];
+        }
+    }
+
+    DarwinSendDDXEvent(kXquartzControllerNotify, 1, AppleWMWindowMenuNotify);
 }
 
 - (void) set_window_menu_check:(NSNumber *)nn
 {
-    NSMenu *menu;
+    NSMenu * const menu = X11App.windowsMenu;
+    NSMenu * const dock_menu = self.dock_menu;
     NSMenuItem *item;
-    int first, count;
-    int n = [nn intValue];
+    int n = nn.intValue;
 
-    menu = [X11App windowsMenu];
-    first = windows_menu_start + 1;
-    count = [menu numberOfItems] - first;
+    NSInteger const count = self.windows_menu_nitems;
+    NSInteger const first = menu.numberOfItems - count;
+
+    int const checked_window_item = self.checked_window_item;
 
     if (checked_window_item >= 0 && checked_window_item < count) {
         item = (NSMenuItem *)[menu itemAtIndex:first + checked_window_item];
@@ -299,7 +300,7 @@ extern char *bundle_id_prefix;
         item = (NSMenuItem *)[dock_menu itemAtIndex:n];
         [item setState:NSOnState];
     }
-    checked_window_item = n;
+    self.checked_window_item = n;
 }
 
 - (void) set_apps_menu:(NSArray *)list
@@ -311,31 +312,29 @@ extern char *bundle_id_prefix;
 #ifdef XQUARTZ_SPARKLE
 - (void) setup_sparkle
 {
-    if (check_for_updates_item)
+    if (self.check_for_updates_item)
         return;  // already did it...
 
-    NSMenu *menu = [x11_about_item menu];
+    NSMenu *menu = [self.x11_about_item menu];
 
-    check_for_updates_item =
-        [menu insertItemWithTitle:NSLocalizedString(
-             @"Check for X11 Updates...",
-             @"Check for X11 Updates...")
-         action:@selector (
-             checkForUpdates:)
-         keyEquivalent:@""
-         atIndex:1];
+    NSMenuItem * const check_for_updates_item =
+        [menu insertItemWithTitle:NSLocalizedString(@"Check for X11 Updates...", @"Check for X11 Updates...")
+                           action:@selector(checkForUpdates:)
+                    keyEquivalent:@""
+                          atIndex:1];
     [check_for_updates_item setTarget:[SUUpdater sharedUpdater]];
     [check_for_updates_item setEnabled:YES];
 
+    self.check_for_updates_item = check_for_updates_item;
+
     // Set X11Controller as the delegate for the updater.
     [[SUUpdater sharedUpdater] setDelegate:self];
 }
 
 // Sent immediately before installing the specified update.
-- (void)updater:(SUUpdater *)updater willInstallUpdate:(SUAppcastItem *)
-   update
+- (void)updater:(SUUpdater *)updater willInstallUpdate:(SUAppcastItem *)update
 {
-    //[self set_can_quit:YES];
+    //self.can_quit = YES;
 }
 
 #endif
@@ -347,10 +346,8 @@ extern char *bundle_id_prefix;
     const char *newargv[4];
     char buf[128];
     char *s;
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
     int stdout_pipe[2];
     int stderr_pipe[2];
-#endif
 
     newargv[0] = [X11App prefs_get_string:@PREFS_LOGIN_SHELL default:"/bin/sh"];
     newargv[1] = "-c";
@@ -363,7 +360,6 @@ extern char *bundle_id_prefix;
         setenv("DISPLAY", buf, TRUE);
     }
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
     if (&asl_log_descriptor) {
         char *asl_sender;
         aslmsg amsg = asl_new(ASL_TYPE_MSG);
@@ -395,7 +391,6 @@ extern char *bundle_id_prefix;
 
         asl_free(amsg);
     }
-#endif
 
     /* Do the fork-twice trick to avoid having to reap zombies */
     child1 = fork();
@@ -413,13 +408,11 @@ extern char *bundle_id_prefix;
             _exit(1);
 
         case 0:                                     /* child2 */
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
             if (&asl_log_descriptor) {
                 /* Replace our stdout/stderr */
                 dup2(stdout_pipe[1], STDOUT_FILENO);
                 dup2(stderr_pipe[1], STDERR_FILENO);
             }
-#endif
 
             /* close all open files except for standard streams */
             max_files = sysconf(_SC_OPEN_MAX);
@@ -442,19 +435,18 @@ extern char *bundle_id_prefix;
         waitpid(child1, &status, 0);
     }
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
     if (&asl_log_descriptor) {
         /* Close the write ends of the pipe */
         close(stdout_pipe[1]);
         close(stderr_pipe[1]);
     }
-#endif
 }
 
 - (void) app_selected:sender
 {
     int tag;
     NSString *item;
+    NSArray * const apps = self.apps;
 
     tag = [sender tag] - 1;
     if (apps == nil || tag < 0 || tag >= [apps count])
@@ -468,12 +460,13 @@ extern char *bundle_id_prefix;
 - (IBAction) apps_table_show:sender
 {
     NSArray *columns;
-    NSMutableArray *oldapps = nil;
+    NSMutableArray *oldapps = self.table_apps;
+    NSTableView * const apps_table = self.apps_table;
 
-    if (table_apps != nil)
-        oldapps = table_apps;
+    NSMutableArray * const table_apps = [[NSMutableArray alloc] initWithCapacity:1];
+    self.table_apps = table_apps;
 
-    table_apps = [[NSMutableArray alloc] initWithCapacity:1];
+    NSArray * const apps = self.apps;
     if (apps != nil)
         [table_apps addObjectsFromArray:apps];
 
@@ -494,6 +487,8 @@ extern char *bundle_id_prefix;
 
 - (IBAction) apps_table_done:sender
 {
+    NSMutableArray * const table_apps = self.table_apps;
+    NSTableView * const apps_table = self.apps_table;
     [apps_table deselectAll:sender];    /* flush edits? */
 
     [self remove_apps_menu];
@@ -504,13 +499,14 @@ extern char *bundle_id_prefix;
 
     [[apps_table window] orderOut:sender];
 
-    [table_apps release];
-    table_apps = nil;
+    self.table_apps = nil;
 }
 
 - (IBAction) apps_table_new:sender
 {
     NSMutableArray *item;
+    NSMutableArray * const table_apps = self.table_apps;
+    NSTableView * const apps_table = self.apps_table;
 
     int row = [apps_table selectedRow], i;
 
@@ -538,6 +534,8 @@ extern char *bundle_id_prefix;
 
 - (IBAction) apps_table_duplicate:sender
 {
+    NSMutableArray * const table_apps = self.table_apps;
+    NSTableView * const apps_table = self.apps_table;
     int row = [apps_table selectedRow], i;
     NSObject *item;
 
@@ -562,6 +560,8 @@ extern char *bundle_id_prefix;
 
 - (IBAction) apps_table_delete:sender
 {
+    NSMutableArray * const table_apps = self.table_apps;
+    NSTableView * const apps_table = self.apps_table;
     int row = [apps_table selectedRow];
 
     if (row >= 0) {
@@ -584,6 +584,7 @@ extern char *bundle_id_prefix;
 
 - (NSInteger) numberOfRowsInTableView:(NSTableView *)tableView
 {
+    NSMutableArray * const table_apps = self.table_apps;
     if (table_apps == nil) return 0;
 
     return [table_apps count];
@@ -592,6 +593,7 @@ extern char *bundle_id_prefix;
 - (id)             tableView:(NSTableView *)tableView
    objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
 {
+    NSMutableArray * const table_apps = self.table_apps;
     NSArray *item;
     int col;
 
@@ -609,6 +611,7 @@ extern char *bundle_id_prefix;
 - (void) tableView:(NSTableView *)tableView setObjectValue:(id)object
     forTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
 {
+    NSMutableArray * const table_apps = self.table_apps;
     NSMutableArray *item;
     int col;
 
@@ -669,11 +672,10 @@ extern char *bundle_id_prefix;
 
 - (IBAction) enable_fullscreen_changed:sender
 {
-    XQuartzRootlessDefault = ![enable_fullscreen intValue];
+    XQuartzRootlessDefault = !self.enable_fullscreen.intValue;
 
-    [enable_fullscreen_menu setEnabled:!XQuartzRootlessDefault];
-    [enable_fullscreen_menu_text setTextColor:XQuartzRootlessDefault ?[
-         NSColor disabledControlTextColor] : [NSColor controlTextColor]];
+    [self.enable_fullscreen_menu setEnabled:!XQuartzRootlessDefault];
+    [self.enable_fullscreen_menu_text setTextColor:XQuartzRootlessDefault ? NSColor.disabledControlTextColor : NSColor.controlTextColor];
 
     DarwinSendDDXEvent(kXquartzSetRootless, 1, XQuartzRootlessDefault);
 
@@ -686,102 +688,85 @@ extern char *bundle_id_prefix;
     DarwinSendDDXEvent(kXquartzToggleFullscreen, 0);
 }
 
-- (void) set_can_quit:(OSX_BOOL)state
-{
-    can_quit = state;
-}
-
 - (IBAction)prefs_changed:sender
 {
     if (!sender)
         return;
 
-    if (sender == fake_buttons) {
-        darwinFakeButtons = [fake_buttons intValue];
+    if (sender == self.fake_buttons) {
+        darwinFakeButtons = self.fake_buttons.intValue;
         [NSApp prefs_set_boolean:@PREFS_FAKEBUTTONS value:darwinFakeButtons];
     }
-    else if (sender == enable_keyequivs) {
-        XQuartzEnableKeyEquivalents = [enable_keyequivs intValue];
+    else if (sender == self.enable_keyequivs) {
+        XQuartzEnableKeyEquivalents = self.enable_keyequivs.intValue;
         [NSApp prefs_set_boolean:@PREFS_KEYEQUIVS value:
          XQuartzEnableKeyEquivalents];
     }
-    else if (sender == sync_keymap) {
-        darwinSyncKeymap = [sync_keymap intValue];
+    else if (sender == self.sync_keymap) {
+        darwinSyncKeymap = self.sync_keymap.intValue;
         [NSApp prefs_set_boolean:@PREFS_SYNC_KEYMAP value:darwinSyncKeymap];
     }
-    else if (sender == enable_fullscreen_menu) {
-        XQuartzFullscreenMenu = [enable_fullscreen_menu intValue];
+    else if (sender == self.enable_fullscreen_menu) {
+        XQuartzFullscreenMenu = self.enable_fullscreen_menu.intValue;
         [NSApp prefs_set_boolean:@PREFS_FULLSCREEN_MENU value:
          XQuartzFullscreenMenu];
     }
-    else if (sender == option_sends_alt) {
+    else if (sender == self.option_sends_alt) {
         BOOL prev_opt_sends_alt = XQuartzOptionSendsAlt;
 
-        XQuartzOptionSendsAlt = [option_sends_alt intValue];
+        XQuartzOptionSendsAlt = self.option_sends_alt.intValue;
         [NSApp prefs_set_boolean:@PREFS_OPTION_SENDS_ALT value:
          XQuartzOptionSendsAlt];
 
         if (prev_opt_sends_alt != XQuartzOptionSendsAlt)
             QuartsResyncKeymap(TRUE);
     }
-    else if (sender == click_through) {
-        [NSApp prefs_set_boolean:@PREFS_CLICK_THROUGH value:[click_through
-                                                             intValue]];
+    else if (sender == self.click_through) {
+        [NSApp prefs_set_boolean:@PREFS_CLICK_THROUGH value:self.click_through.intValue];
     }
-    else if (sender == focus_follows_mouse) {
-        [NSApp prefs_set_boolean:@PREFS_FFM value:[focus_follows_mouse
-                                                   intValue]];
+    else if (sender == self.focus_follows_mouse) {
+        [NSApp prefs_set_boolean:@PREFS_FFM value:self.focus_follows_mouse.intValue];
     }
-    else if (sender == focus_on_new_window) {
-        [NSApp prefs_set_boolean:@PREFS_FOCUS_ON_NEW_WINDOW value:[
-             focus_on_new_window intValue]];
+    else if (sender == self.focus_on_new_window) {
+        [NSApp prefs_set_boolean:@PREFS_FOCUS_ON_NEW_WINDOW value:self.focus_on_new_window.intValue];
     }
-    else if (sender == enable_auth) {
-        [NSApp prefs_set_boolean:@PREFS_NO_AUTH value:![enable_auth intValue]
-        ];
+    else if (sender == self.enable_auth) {
+        [NSApp prefs_set_boolean:@PREFS_NO_AUTH value:!self.enable_auth.intValue];
     }
-    else if (sender == enable_tcp) {
-        [NSApp prefs_set_boolean:@PREFS_NO_TCP value:![enable_tcp intValue]];
+    else if (sender == self.enable_tcp) {
+        [NSApp prefs_set_boolean:@PREFS_NO_TCP value:!self.enable_tcp.intValue];
     }
-    else if (sender == depth) {
-        [NSApp prefs_set_integer:@PREFS_DEPTH value:[depth selectedTag]];
+    else if (sender == self.depth) {
+        [NSApp prefs_set_integer:@PREFS_DEPTH value:self.depth.selectedTag];
     }
-    else if (sender == sync_pasteboard) {
-        BOOL pbproxy_active = [sync_pasteboard intValue];
+    else if (sender == self.sync_pasteboard) {
+        BOOL pbproxy_active = self.sync_pasteboard.intValue;
         [NSApp prefs_set_boolean:@PREFS_SYNC_PB value:pbproxy_active];
 
-        [sync_pasteboard_to_clipboard setEnabled:pbproxy_active];
-        [sync_pasteboard_to_primary setEnabled:pbproxy_active];
-        [sync_clipboard_to_pasteboard setEnabled:pbproxy_active];
-        [sync_primary_immediately setEnabled:pbproxy_active];
+        [self.sync_pasteboard_to_clipboard setEnabled:pbproxy_active];
+        [self.sync_pasteboard_to_primary setEnabled:pbproxy_active];
+        [self.sync_clipboard_to_pasteboard setEnabled:pbproxy_active];
+        [self.sync_primary_immediately setEnabled:pbproxy_active];
 
         // setEnabled doesn't do this...
-        [sync_text1 setTextColor:pbproxy_active ?[NSColor controlTextColor] :
-         [NSColor disabledControlTextColor]];
-        [sync_text2 setTextColor:pbproxy_active ?[NSColor controlTextColor] :
-         [NSColor disabledControlTextColor]];
+        [self.sync_text1 setTextColor:pbproxy_active ? NSColor.controlTextColor : NSColor.disabledControlTextColor];
+        [self.sync_text2 setTextColor:pbproxy_active ? NSColor.controlTextColor : NSColor.disabledControlTextColor];
     }
-    else if (sender == sync_pasteboard_to_clipboard) {
-        [NSApp prefs_set_boolean:@PREFS_SYNC_PB_TO_CLIPBOARD value:[
-             sync_pasteboard_to_clipboard intValue]];
+    else if (sender == self.sync_pasteboard_to_clipboard) {
+        [NSApp prefs_set_boolean:@PREFS_SYNC_PB_TO_CLIPBOARD value:self.sync_pasteboard_to_clipboard.intValue];
     }
-    else if (sender == sync_pasteboard_to_primary) {
-        [NSApp prefs_set_boolean:@PREFS_SYNC_PB_TO_PRIMARY value:[
-             sync_pasteboard_to_primary intValue]];
+    else if (sender == self.sync_pasteboard_to_primary) {
+        [NSApp prefs_set_boolean:@PREFS_SYNC_PB_TO_PRIMARY value:self.sync_pasteboard_to_primary.intValue];
     }
-    else if (sender == sync_clipboard_to_pasteboard) {
-        [NSApp prefs_set_boolean:@PREFS_SYNC_CLIPBOARD_TO_PB value:[
-             sync_clipboard_to_pasteboard intValue]];
+    else if (sender == self.sync_clipboard_to_pasteboard) {
+        [NSApp prefs_set_boolean:@PREFS_SYNC_CLIPBOARD_TO_PB value:self.sync_clipboard_to_pasteboard.intValue];
     }
-    else if (sender == sync_primary_immediately) {
-        [NSApp prefs_set_boolean:@PREFS_SYNC_PRIMARY_ON_SELECT value:[
-             sync_primary_immediately intValue]];
+    else if (sender == self.sync_primary_immediately) {
+        [NSApp prefs_set_boolean:@PREFS_SYNC_PRIMARY_ON_SELECT value:self.sync_primary_immediately.intValue];
     }
-    else if (sender == scroll_in_device_direction) {
-        XQuartzScrollInDeviceDirection =
-            [scroll_in_device_direction intValue];
-        [NSApp prefs_set_boolean:@PREFS_SCROLL_IN_DEV_DIRECTION value:
-         XQuartzScrollInDeviceDirection];
+    else if (sender == self.scroll_in_device_direction) {
+        XQuartzScrollInDeviceDirection = self.scroll_in_device_direction.intValue;
+        [NSApp prefs_set_boolean:@PREFS_SCROLL_IN_DEV_DIRECTION value:XQuartzScrollInDeviceDirection];
     }
 
     [NSApp prefs_synchronize];
@@ -794,71 +779,42 @@ extern char *bundle_id_prefix;
     BOOL pbproxy_active =
         [NSApp prefs_get_boolean:@PREFS_SYNC_PB default:YES];
 
-    // Remove preferences from the GUI which are not supported
-    // TODO: Change 1117 to NSAppKitVersionNumber10_7 when it is defined
-    if (scroll_in_device_direction && NSAppKitVersionNumber < 1117) {
-        [scroll_in_device_direction removeFromSuperview];
-        scroll_in_device_direction = nil;
-    }
-    else {
-        [scroll_in_device_direction setIntValue:
-         XQuartzScrollInDeviceDirection];
-    }
+    [self.scroll_in_device_direction setIntValue:XQuartzScrollInDeviceDirection];
 
-    [fake_buttons setIntValue:darwinFakeButtons];
-    [enable_keyequivs setIntValue:XQuartzEnableKeyEquivalents];
-    [sync_keymap setIntValue:darwinSyncKeymap];
-    [option_sends_alt setIntValue:XQuartzOptionSendsAlt];
-    [click_through setIntValue:[NSApp prefs_get_boolean:@PREFS_CLICK_THROUGH
-                                default:NO]];
-    [focus_follows_mouse setIntValue:[NSApp prefs_get_boolean:@PREFS_FFM
-                                      default:NO]];
-    [focus_on_new_window setIntValue:[NSApp prefs_get_boolean:
-                                      @PREFS_FOCUS_ON_NEW_WINDOW default:YES]
-    ];
+    [self.fake_buttons setIntValue:darwinFakeButtons];
+    [self.enable_keyequivs setIntValue:XQuartzEnableKeyEquivalents];
+    [self.sync_keymap setIntValue:darwinSyncKeymap];
+    [self.option_sends_alt setIntValue:XQuartzOptionSendsAlt];
+    [self.click_through setIntValue:[NSApp prefs_get_boolean:@PREFS_CLICK_THROUGH default:NO]];
+    [self.focus_follows_mouse setIntValue:[NSApp prefs_get_boolean:@PREFS_FFM default:NO]];
+    [self.focus_on_new_window setIntValue:[NSApp prefs_get_boolean:@PREFS_FOCUS_ON_NEW_WINDOW default:YES]];
 
-    [enable_auth setIntValue:![NSApp prefs_get_boolean:@PREFS_NO_AUTH default
-                               :NO]];
-    [enable_tcp setIntValue:![NSApp prefs_get_boolean:@PREFS_NO_TCP default:
-                              NO]];
+    [self.enable_auth setIntValue:![NSApp prefs_get_boolean:@PREFS_NO_AUTH default:NO]];
+    [self.enable_tcp setIntValue:![NSApp prefs_get_boolean:@PREFS_NO_TCP default:NO]];
 
-    [depth selectItemAtIndex:[depth indexOfItemWithTag:[NSApp
-                                                        prefs_get_integer:
-                                                        @PREFS_DEPTH default:
-                                                        -1]]];
+    [self.depth selectItemAtIndex:[self.depth indexOfItemWithTag:[NSApp prefs_get_integer:@PREFS_DEPTH default:-1]]];
 
-    [sync_pasteboard setIntValue:pbproxy_active];
-    [sync_pasteboard_to_clipboard setIntValue:[NSApp prefs_get_boolean:
-                                               @PREFS_SYNC_PB_TO_CLIPBOARD
-                                               default:YES]];
-    [sync_pasteboard_to_primary setIntValue:[NSApp prefs_get_boolean:
-                                             @PREFS_SYNC_PB_TO_PRIMARY
-                                             default:YES]];
-    [sync_clipboard_to_pasteboard setIntValue:[NSApp prefs_get_boolean:
-                                               @PREFS_SYNC_CLIPBOARD_TO_PB
-                                               default:YES]];
-    [sync_primary_immediately setIntValue:[NSApp prefs_get_boolean:
-                                           @PREFS_SYNC_PRIMARY_ON_SELECT
-                                           default:NO]];
+    [self.sync_pasteboard setIntValue:pbproxy_active];
+    [self.sync_pasteboard_to_clipboard setIntValue:[NSApp prefs_get_boolean:@PREFS_SYNC_PB_TO_CLIPBOARD default:YES]];
+    [self.sync_pasteboard_to_primary setIntValue:[NSApp prefs_get_boolean:@PREFS_SYNC_PB_TO_PRIMARY default:YES]];
+    [self.sync_clipboard_to_pasteboard setIntValue:[NSApp prefs_get_boolean:@PREFS_SYNC_CLIPBOARD_TO_PB default:YES]];
+    [self.sync_primary_immediately setIntValue:[NSApp prefs_get_boolean:@PREFS_SYNC_PRIMARY_ON_SELECT default:NO]];
 
-    [sync_pasteboard_to_clipboard setEnabled:pbproxy_active];
-    [sync_pasteboard_to_primary setEnabled:pbproxy_active];
-    [sync_clipboard_to_pasteboard setEnabled:pbproxy_active];
-    [sync_primary_immediately setEnabled:pbproxy_active];
+    [self.sync_pasteboard_to_clipboard setEnabled:pbproxy_active];
+    [self.sync_pasteboard_to_primary setEnabled:pbproxy_active];
+    [self.sync_clipboard_to_pasteboard setEnabled:pbproxy_active];
+    [self.sync_primary_immediately setEnabled:pbproxy_active];
 
     // setEnabled doesn't do this...
-    [sync_text1 setTextColor:pbproxy_active ?[NSColor controlTextColor] : [
-         NSColor disabledControlTextColor]];
-    [sync_text2 setTextColor:pbproxy_active ?[NSColor controlTextColor] : [
-         NSColor disabledControlTextColor]];
+    [self.sync_text1 setTextColor:pbproxy_active ? NSColor.controlTextColor : NSColor.disabledControlTextColor];
+    [self.sync_text2 setTextColor:pbproxy_active ? NSColor.controlTextColor : NSColor.disabledControlTextColor];
 
-    [enable_fullscreen setIntValue:!XQuartzRootlessDefault];
-    [enable_fullscreen_menu setIntValue:XQuartzFullscreenMenu];
-    [enable_fullscreen_menu setEnabled:!XQuartzRootlessDefault];
-    [enable_fullscreen_menu_text setTextColor:XQuartzRootlessDefault ?[
-         NSColor disabledControlTextColor] : [NSColor controlTextColor]];
+    [self.enable_fullscreen setIntValue:!XQuartzRootlessDefault];
+    [self.enable_fullscreen_menu setIntValue:XQuartzFullscreenMenu];
+    [self.enable_fullscreen_menu setEnabled:!XQuartzRootlessDefault];
+    [self.enable_fullscreen_menu_text setTextColor:XQuartzRootlessDefault ? NSColor.disabledControlTextColor : NSColor.controlTextColor];
 
-    [prefs_panel makeKeyAndOrderFront:sender];
+    [self.prefs_panel makeKeyAndOrderFront:sender];
 }
 
 - (IBAction) quit:sender
@@ -868,23 +824,18 @@ extern char *bundle_id_prefix;
 
 - (IBAction) x11_help:sender
 {
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
-    AHLookupAnchor((CFStringRef)NSLocalizedString(@"Mac Help",
-                                                  no comment),
-                   CFSTR("mchlp2276"));
-#else
     AHLookupAnchor(CFSTR("com.apple.machelp"), CFSTR("mchlp2276"));
-#endif
 }
 
 - (OSX_BOOL) validateMenuItem:(NSMenuItem *)item
 {
     NSMenu *menu = [item menu];
+    NSMenu * const dock_menu = self.dock_menu;
 
-    if (item == toggle_fullscreen_item)
+    if (item == self.toggle_fullscreen_item)
         return !XQuartzIsRootless;
     else if (menu == [X11App windowsMenu] || menu == dock_menu
-             || (menu == [x11_about_item menu] && [item tag] == 42))
+             || (menu == [self.x11_about_item menu] && [item tag] == 42))
         return (AppleWMSelectedEvents() & AppleWMControllerNotifyMask) != 0;
     else
         return TRUE;
@@ -912,7 +863,7 @@ extern char *bundle_id_prefix;
     NSString *msg;
     NSString *title;
 
-    if (can_quit ||
+    if (self.can_quit ||
         [X11App prefs_get_boolean:@PREFS_NO_QUIT_ALERT default:NO])
         return NSTerminateNow;
 
@@ -951,26 +902,26 @@ extern char *bundle_id_prefix;
 {
     x_list *node;
 
-    finished_launching = YES;
+    self.finished_launching = YES;
 
-    for (node = pending_apps; node != NULL; node = node->next) {
+    for (node = self.pending_apps; node != NULL; node = node->next) {
         NSString *filename = node->data;
         [self launch_client:filename];
         [filename release];
     }
 
-    x_list_free(pending_apps);
-    pending_apps = NULL;
+    x_list_free(self.pending_apps);
+    self.pending_apps = NULL;
 }
 
 - (OSX_BOOL) application:(NSApplication *)app openFile:(NSString *)filename
 {
     const char *name = [filename UTF8String];
 
-    if (finished_launching)
+    if (self.finished_launching)
         [self launch_client:filename];
     else if (name[0] != ':')            /* ignore display names */
-        pending_apps = x_list_prepend(pending_apps, [filename retain]);
+        self.pending_apps = x_list_prepend(self.pending_apps, [filename retain]);
 
     /* FIXME: report failures. */
     return YES;
